﻿namespace U2UConsult.WinRT.Controls
{
    using Windows.UI.Xaml;
    using Windows.UI.Xaml.Controls;
    using Windows.UI.Xaml.Controls.Primitives;
    using Windows.UI.Xaml.Shapes;

    [TemplatePart(Name = ThumbPartName, Type = typeof(Thumb))]
    [TemplatePart(Name = RectanglePartName, Type = typeof(Rectangle))]
    public class SimpleSlider : Control
    {
        private const string ThumbPartName = "PART_Thumb";
        private const string RectanglePartName = "PART_Rectangle";

        private Thumb thumb;
        private Rectangle rectangle;

        // Using a DependencyProperty as the backing store for Minimum.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MinimumProperty =
            DependencyProperty.Register("Minimum", "Object", typeof(SimpleSlider).FullName, new PropertyMetadata(0.0));

        // Using a DependencyProperty as the backing store for Maximum.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MaximumProperty =
            DependencyProperty.Register("Maximum", "Object", typeof(SimpleSlider).FullName, new PropertyMetadata(0.0));

        // Using a DependencyProperty as the backing store for Value.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ValueProperty =
            DependencyProperty.Register("Value", "Object", typeof(SimpleSlider).FullName, new PropertyMetadata(0.0, OnValueChanged));

        /// <summary>
        /// Gets or sets the minimum.
        /// </summary>
        public double Minimum
        {
            get { return (double)GetValue(MinimumProperty); }
            set { SetValue(MinimumProperty, value); }
        }

        /// <summary>
        /// Gets or sets the maximum.
        /// </summary>
        public double Maximum
        {
            get { return (double)GetValue(MaximumProperty); }
            set { SetValue(MaximumProperty, value); }
        }

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        public double Value
        {
            get { return (double)GetValue(ValueProperty); }
            set
            {
                SetValue(ValueProperty, value);
            }
        }

        /// <summary>
        /// When overridden in a derived class, is invoked whenever application code or internal processes call <see cref="M:System.Windows.FrameworkElement.ApplyTemplate"/>.
        /// </summary>
        protected override void OnApplyTemplateCore()
        {
            base.OnApplyTemplateCore();

            this.thumb = this.GetTemplateChild(ThumbPartName) as Thumb;
            if (this.thumb != null)
            {
                this.thumb.DragDelta += this.Thumb_DragDelta;
            }

            this.rectangle = this.GetTemplateChild(RectanglePartName) as Rectangle;

            this.SizeChanged += new SizeChangedEventHandler(SimpleSlider_SizeChanged);
        }

        /// <summary>
        /// Called when size changed.
        /// </summary>
        private void SimpleSlider_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            if (e.NewSize.Width != e.PreviousSize.Width)
            {
                this.UpdateControls();
            }
        }

        /// <summary>
        /// Called when value changed.
        /// </summary>
        private static void OnValueChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            // Simple version: no coercion involved.
            var customSlider = (SimpleSlider)dependencyObject;
            customSlider.UpdateControls();
        }

        /// <summary>
        /// Handles the DragDelta event of the Thumb control.
        /// </summary>
        private void Thumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            var pixelDiff = e.HorizontalChange;
            var currentLeft = Canvas.GetLeft(this.thumb);

            // trying to drag too far left
            if ((currentLeft + pixelDiff) < 0)
            {
                this.Value = 0;
            }
            // trying to drag too far right
            else if ((currentLeft + pixelDiff + this.thumb.ActualWidth) > this.ActualWidth)
            {
                this.Value = this.Maximum;
            }
            else
            {
                var totalSize = this.ActualWidth;
                var ratioDiff = pixelDiff / totalSize;
                var rangeSize = this.Maximum - this.Minimum;
                var rangeDiff = rangeSize * ratioDiff;
                this.Value += rangeDiff;
            }
        }

        /// <summary>
        /// Updates the controls.
        /// </summary>
        private void UpdateControls()
        {
            double halfTheThumbWith = 0;

            if (this.thumb != null)
            {
                halfTheThumbWith = this.thumb.ActualWidth / 2;
            }

            double totalSize = this.ActualWidth - halfTheThumbWith * 2;

            double ratio = totalSize / (this.Maximum - this.Minimum);

            if (this.thumb != null)
            {
                Canvas.SetLeft(this.thumb, ratio * this.Value);
            }

            if (this.rectangle != null)
            {
                this.rectangle.Width = ratio * this.Value + halfTheThumbWith;
            }
        }
    }
}
